/*
 *  COOL TITLE GENERATOR
 *
 *  cooltitle-calcwidth.c:  Use FreeType library to guestimate width of text.
 *
 *  Copyright (C) 2000  Jim Evins <evins@snaught.com>.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

#include <stdio.h>

/*********************************************************************/
/* Uncomment the appropriate directive for freetype on your platform */
/*********************************************************************/
/*#include <freetype.h>*/       /* 1.2, as distributed w/ RH 6.1 */
#include <freetype/freetype.h>  /* 1.3, as distributed w/ RH 6.2 */
/*********************************************************************/

void usage( char *prog );

int main( int argc, char *argv[] )
{
  int                 imap, i, n;
  char                *fontfile, *string;
  TT_Engine           engine;
  TT_Face             face;
  TT_Face_Properties  properties;
  TT_Instance         instance;
  TT_Glyph            glyph;
  TT_Glyph_Metrics    metrics;
  TT_UShort           platformID, encodingID;
  TT_CharMap          charmap;
  TT_UShort           *glyph_string;
  TT_UShort           flags;
  TT_FWord            x, x1, x2, x_min, x_max; /* TrueType Funits */
  float               units_per_em;

  /* Parse arguments */
  if ( argc != 3 ) {
    usage( argv[0] );
  }
  fontfile = argv[1];
  string   = argv[2];

  /* Initialize FreeType engine, open font file and create objects */
  if ( TT_Init_FreeType( &engine ) != TT_Err_Ok ) {
    fprintf( stderr, "Error initializing FreeType engine\n" );
    exit( -1 );
  }
  if ( TT_Open_Face( engine, fontfile, &face ) != TT_Err_Ok ) {
    fprintf( stderr, "Error opening font file \"%s\"\n",
	     fontfile );
    exit( -1 );
  }
  if ( TT_New_Instance( face, &instance ) != TT_Err_Ok ) {
    fprintf( stderr, "Error creating instance for \"%s\"\n",
	     fontfile );
    exit( -1 );
  }
  if ( TT_New_Glyph( face, &glyph ) != TT_Err_Ok ) {
    fprintf( stderr, "Error creating glyph container\n" );
    exit( -1 );
  }

  /* Get face properties */
  TT_Get_Face_Properties( face, &properties );
  units_per_em = (float)properties.header->Units_Per_EM;

  /* Extract Unicode/Latin-1 character/glyph map */
  for ( imap=0; imap<properties.num_CharMaps; imap++ ) {
    TT_Get_CharMap_ID( face, imap, &platformID, &encodingID );
    if ( (platformID == 3) && (encodingID == 1) /* Unicode/Latin-1 */ ) {
      TT_Get_CharMap( face, imap, &charmap );
      break;
    }
  }
  if ( imap == properties.num_CharMaps ) {
    fprintf( stderr, "Fontfile does not contain Unicode/Latin-1 map\n" );
    exit( -1 );
  }

  /* Convert from string of ASCII codes to string of glyph codes */
  n = strlen( string );
  glyph_string = (TT_UShort *)malloc( n * sizeof(TT_UShort) );
  for ( i=0; i<n; i++ ) {
    glyph_string[i] = TT_Char_Index( charmap, (TT_UShort)string[i] );
  }

  /* Find horizontal extent of string */
  flags = 0;
  x = x_min = x_max = 0;
  for ( i=0; i<n; i++ ) {
    TT_Load_Glyph( instance, glyph, glyph_string[i], flags );
    TT_Get_Glyph_Metrics( glyph, &metrics );
    x1 = x + metrics.bbox.xMin;
    x2 = x + metrics.bbox.xMax;
    if ( x1 < x_min ) x_min = x1;
    if ( x2 > x_max ) x_max = x2;
    x += metrics.advance;
  }

  /* Final output */
  printf( "%f\n", (x_max-x_min)/units_per_em );

  /* Close up shop */
  TT_Close_Face( face );
  TT_Done_FreeType( engine );

}


void usage( char *prog )
{
  fprintf( stderr, "Usage:\n" );
  fprintf( stderr, " %s fontfile string\n", prog );
}

